@@ -2,11 +2,13 @@
 
 # Core
 import os
+import time
 
 # BrainGlobe
-from bg_atlasapi import BrainGlobeAtlas
-from bg_atlasapi.list_atlases import get_all_atlases_lastversions, get_downloaded_atlases, get_local_atlas_version
-from bg_atlasapi.utils import check_internet_connection
+from brainglobe_atlasapi import BrainGlobeAtlas
+from brainglobe_atlasapi.list_atlases import get_all_atlases_lastversions, get_downloaded_atlases, \
+    get_local_atlas_version
+from brainglobe_atlasapi.utils import check_internet_connection
 
 # PyImageJ / Scyjava
 from scyjava import jimport
@@ -14,6 +16,38 @@ import imagej
 
 # JPype
 from jpype.types import JString, JArray
+#
+from typing import Literal
+
+LogLevel = Literal['OFF', 'DEBUG', 'INFO']
+
+atlases_using_allen_ccfv3_convention = [
+        'example_mouse_100um',
+        'allen_mouse_10um',
+        'allen_mouse_25um',
+        'allen_mouse_50um',
+        'allen_mouse_100um',
+        'kim_mouse_10um',
+        'kim_mouse_25um',
+        'kim_mouse_50um',
+        'kim_mouse_100um',
+        'osten_mouse_10um',
+        'osten_mouse_25um',
+        'osten_mouse_50um',
+        'osten_mouse_100um',
+        'perens_lsfm_mouse_20um',
+        'kim_dev_mouse_stp_10um',
+        'kim_dev_mouse_idisco_10um',
+        'kim_dev_mouse_mri_a0_10um',
+        'kim_dev_mouse_mri_adc_10um',
+        'kim_dev_mouse_mri_dwi_10um',
+        'kim_dev_mouse_mri_fa_10um',
+        'kim_dev_mouse_mri_mtr_10um',
+        'kim_dev_mouse_mri_t2_10um',
+        'allen_mouse_bluebrain_barrels_10um',
+        'allen_mouse_bluebrain_barrels_25um',
+        'princeton_mouse_20um'
+    ]
 
 def get_java_dependencies():
     """
@@ -22,12 +56,35 @@ def get_java_dependencies():
     these jars should be available in https://maven.scijava.org/
     :return:
     """
-    imagej_core_dep = 'net.imagej:imagej:2.14.0'
-    imagej_legacy_dep = 'net.imagej:imagej-legacy:1.2.1'
-    abba_dep = 'ch.epfl.biop:ImageToAtlasRegister:0.8.0'
-    return [imagej_core_dep, imagej_legacy_dep, abba_dep]
+    return ['net.imagej:imagej:2.16.0',
+            'net.imagej:imagej-legacy:2.0.0',
+            'ch.epfl.biop:ijl-utilities-wrappers:0.11.2',
+            'ch.epfl.biop:ImageToAtlasRegister:0.10.3',
+            'ch.epfl.biop:bigdataviewer-biop-tools:0.11.0',
+            'sc.fiji:bigdataviewer-playground:0.11.1',
+            'sc.fiji.bigdataviewer:bigdataviewer-playground-display:0.5.0',
+            'sc.fiji:bigwarp_fiji:9.3.0',
+            'net.imglib2:imglib2-realtransform:4.0.3',
+            'com.formdev:flatlaf:3.5.1',
+            'ch.epfl.biop:bigdataviewer-image-loaders:0.9.0',
+            'ch.epfl.biop:atlas:0.3.1',
+            'org.scijava:scijava-ui-swing:1.0.3',
+            'net.imglib2:imglib2:7.1.2',
+            'org.janelia.saalfeldlab:n5:3.3.0',
+            'org.janelia.saalfeldlab:n5-blosc:1.1.1',
+            'org.janelia.saalfeldlab:n5-ij:4.2.5',
+            # 'org.janelia.saalfeldlab:n5 - imglib2:',
+            'org.janelia.saalfeldlab:n5-aws-s3:4.2.1',
+            'org.janelia.saalfeldlab:n5-google-cloud:4.1.1',
+            'org.janelia.saalfeldlab:n5-viewer_fiji:6.1.2',
+            'org.janelia.saalfeldlab:n5-zarr:1.3.5',
+            'org.janelia.saalfeldlab:n5-universe:1.6.0',
+            ]
 
-def start_imagej(headless: bool = False):
+
+
+def start_imagej(headless: bool = False,
+                 log_level: LogLevel = 'INFO'):
     mode = "headless"
     if not headless:
         mode = "interactive"
@@ -37,11 +94,8 @@ def start_imagej(headless: bool = False):
     from scyjava import jimport
     from jpype.types import JString
 
-    # loci.common.DebugTools.enableLogging("OFF");
     DebugTools = jimport('loci.common.DebugTools')
-    # DebugTools.enableLogging('OFF')
-    DebugTools.enableLogging("INFO")
-    # DebugTools.enableLogging("DEBUG")
+    DebugTools.enableLogging(JString(log_level))
 
     import platform
     if platform.system() == 'Windows':
@@ -55,8 +109,7 @@ def start_imagej(headless: bool = False):
         try:
             print('Attempt to set ABBA Atlas cache directory to ' + directory)
             os.makedirs(directory, exist_ok=True)
-            atlasPath = str(directory)
-            AtlasLocationHelper.defaultCacheDir = File(JString(atlasPath))
+            AtlasLocationHelper.defaultCacheDir = File(JString(str(directory)))
             print('ABBA Atlas cache directory set to ' + directory)
         except OSError:
             print('ERROR! Could not set ABBA Atlas cache dir')
@@ -69,6 +122,13 @@ def start_imagej(headless: bool = False):
     if not headless:
         ij.ui().showUI()
 
+    # Adds Python information to ABBA help command
+    ABBAForumHelpCommand = jimport('ch.epfl.biop.atlas.aligner.command.ABBAForumHelpCommand')
+    from importlib.metadata import version
+    python_info = 'ABBA Python ' + str(version("abba-python"))
+    ABBAForumHelpCommand.pythonInformation = JString(python_info)
+
+
 def add_brainglobe_atlases(ij):
     # TODO : check connection available or not
     try:
@@ -83,7 +143,7 @@ def add_brainglobe_atlases(ij):
 
     AtlasChooserCommand = jimport('ch.epfl.biop.atlas.scijava.AtlasChooserCommand')
 
-    from jpype import JImplements, JOverride
+    from jpype import JImplements, JOverride, JString
     Supplier = jimport('java.util.function.Supplier')
 
     # initialized
@@ -98,7 +158,8 @@ def add_brainglobe_atlases(ij):
         @JOverride
         def get(self):
             bg_atlas = BrainGlobeAtlas(self.atlas_name)
-            from abba_python.abba_private.abba_atlas import AbbaAtlas  # but wth ????
+            from abba_python.abba_atlas import AbbaAtlas
+            # from
             current_atlas = AbbaAtlas(bg_atlas, self.ij)
             current_atlas.initialize(None, None)
             Abba.opened_atlases[self.atlas_name] = current_atlas
@@ -107,6 +168,14 @@ def add_brainglobe_atlases(ij):
     for atlas_name in available_atlases.keys():
         AtlasChooserCommand.registerAtlas(atlas_name, AtlasSupplier(atlas_name, ij))
 
+    # Specify which atlases are compatible with DeepSlice Mouse and Rat models - this only affects the GUI
+    DeepSliceHelper = jimport('ch.epfl.biop.atlas.aligner.DeepSliceHelper')
+
+    for atlas_name in atlases_using_allen_ccfv3_convention:
+        DeepSliceHelper.addMouseCompatibleAtlas(JString(atlas_name))
+
+    DeepSliceHelper.addRatCompatibleAtlas(JString('whs_sd_rat_39um'))
+
 
 class Abba:
     """Abba object which can be used to register sections to a BrainGlobe atlas object
@@ -126,22 +195,30 @@ class Abba:
 
         abba_python.ij() returns the current ij instance, which can also be reused in another
         abba_python instance
-
-    slicing_mode :
-        should be 'coronal', 'sagittal' or 'horizontal'
-        TO IMPROVE : test how well this matches with the BrainGlobe API
+    x_axis :
+        See https://github.com/BIOP/ijp-imagetoatlas/blob/master/src/main/java/ch/epfl/biop/atlas/aligner/command/ABBAStartCommand.java
+        should be 'AP', 'PA', 'LR', 'RL', 'DV', 'VD'
+    y_axis :
+        should be 'AP', 'PA', 'LR', 'RL', 'DV', 'VD'
+    z_axis :
+        should be 'AP', 'PA', 'LR', 'RL', 'DV', 'VD'
+    log_level :
+        should be taken within LogLevel literal
 
     """
 
     opened_atlases: dict = {}
 
     def __init__(
-        self,
-        atlas_name: str = 'Adult Mouse Brain - Allen Brain Atlas V3',
-        ij=None,
-        slicing_mode: str = 'coronal',  # or sagittal or horizontal
-        headless: bool = False,
-        print_config: bool = True
+            self,
+            atlas_name: str = 'Adult Mouse Brain - Allen Brain Atlas V3p1',
+            ij=None,
+            x_axis: str = 'RL',
+            y_axis: str = 'SI',
+            z_axis: str = 'AP',
+            headless: bool = False,
+            print_config: bool = True,
+            log_level: LogLevel = 'INFO'
     ):
         if ij is None:
             if headless:
@@ -156,7 +233,7 @@ class Abba:
 
         # Look in object service to see if the atlas is not already opened by any chance
         # in java TODO
-
+        # return
         # or in python
         if atlas_name not in Abba.opened_atlases:
             if atlas_name == 'Adult Mouse Brain - Allen Brain Atlas V3':
@@ -175,22 +252,29 @@ class Abba:
                     'ch.epfl.biop.atlas.rat.waxholm.spraguedawley.v4p2.command.WaxholmSpragueDawleyRatV4p2Command')
                 atlas = ij.command().run(WaxholmSpragueDawleyRatV4Command, True).get().getOutput("ba")
                 Abba.opened_atlases[atlas_name] = atlas
+            elif atlas_name == 'allen_mouse_10um_java':
+                AllenBrainAdultMouseAtlasCCF2017v3p1ASRCommand = jimport(
+                    'ch.epfl.biop.atlas.mouse.allen.ccfv3p1asr.command.AllenBrainAdultMouseAtlasCCF2017v3p1ASRCommand')
+                atlas = ij.command().run(AllenBrainAdultMouseAtlasCCF2017v3p1ASRCommand, True).get().getOutput("ba")
+                Abba.opened_atlases[atlas_name] = atlas
             else:
                 bg_atlas = BrainGlobeAtlas(atlas_name)
                 # initialized
-                from abba_private import abba_atlas
-                atlas = abba_atlas.AbbaAtlas(bg_atlas, ij)
+                from abba_python.abba_atlas import AbbaAtlas
+                atlas = AbbaAtlas(bg_atlas, ij)
                 atlas.initialize(None, None)
                 Abba.opened_atlases[atlas_name] = atlas
                 ij.object().addObject(atlas, atlas_name)  # store it in java's object service
 
         self.atlas = Abba.opened_atlases[atlas_name]
-        self.slicing_mode = slicing_mode
+        self.x_axis = x_axis
+        self.y_axis = y_axis
+        self.z_axis = z_axis
         self.atlas_name = atlas_name
 
-        # .. but before : logger, please shut up
+        # Setting logging options
         DebugTools = jimport('loci.common.DebugTools')
-        DebugTools.enableLogging('OFF')
+        DebugTools.enableLogging(JString(log_level))
 
         # Ok, let's create abba_python's model: mp = multipositioner
 
@@ -200,7 +284,9 @@ class Abba:
             self.print_config()
 
         self.mp = ij.command().run(ABBAStartCommand, True,
-                                   'slicing_mode', self.slicing_mode,
+                                   'x_axis', self.x_axis,
+                                   'y_axis', self.y_axis,
+                                   'z_axis', self.z_axis,
                                    'ba', self.atlas
                                    ).get().getOutput('mp')
 
@@ -279,11 +365,35 @@ class Abba:
         """
         if not hasattr(self, 'bdv_view'):
             # no bdv view properties : creates a new one
-            BdvMultislicePositionerView = jimport('ch.epfl.biop.atlas.aligner.gui.bdv.BdvMultislicePositionerView')
-            DefaultBdvSupplier = jimport('sc.fiji.bdvpg.bdv.supplier.DefaultBdvSupplier')
-            SerializableBdvOptions = jimport('sc.fiji.bdvpg.bdv.supplier.SerializableBdvOptions')
-            bdvh = DefaultBdvSupplier(SerializableBdvOptions()).get()
-            self.bdv_view = BdvMultislicePositionerView(self.mp, bdvh)
+            SwingUtilities = jimport('javax.swing.SwingUtilities')
+            Runnable = jimport('java.lang.Runnable')
+
+            from jpype import JImplements, JOverride
+
+            # We need to create the view in the Java UI thread
+            @JImplements(Runnable)
+            class BdvViewGetter:
+                def __init__(self, mp):
+                    self.bdv_view = None
+                    self.mp = mp
+
+                @JOverride
+                def run(self):
+                    DefaultBdvSupplier = jimport('sc.fiji.bdvpg.bdv.supplier.DefaultBdvSupplier')
+                    SerializableBdvOptions = jimport('sc.fiji.bdvpg.bdv.supplier.SerializableBdvOptions')
+                    BdvMultislicePositionerView = jimport(
+                        'ch.epfl.biop.atlas.aligner.gui.bdv.BdvMultislicePositionerView')
+                    bdvh = DefaultBdvSupplier(SerializableBdvOptions()).get()
+                    self.bdv_view = BdvMultislicePositionerView(self.mp, bdvh)
+
+                def get_bdv_view(self):
+                    return self.bdv_view
+
+            bdv_view_getter = BdvViewGetter(self.mp)
+            SwingUtilities.invokeLater(bdv_view_getter)
+            while bdv_view_getter.get_bdv_view() is None:
+                time.sleep(0.2)
+            self.bdv_view = bdv_view_getter.get_bdv_view()
         else:
             # TODO: make sure it is visible
             pass
@@ -358,6 +468,42 @@ class Abba:
         self.mp.waitForTasks()
 
     # ---------------------- AUTOGENERATED CODE with the main method from ScijavaCommandToPython
+    def benchmark(self,
+                   comment: str,
+                   demo_dataset: str,
+                   use_gui: bool,
+                   wait_betweem_each_step: bool):
+        """
+        Check for updates
+
+        Parameters:
+        comment (str):
+        demo_dataset (str):
+        use_gui (bool): Use graphical user interface
+        wait_betweem_each_step (bool):
+        """
+        ABBABenchMarkCommand = jimport('ch.epfl.biop.atlas.aligner.command.ABBABenchMarkCommand')
+        return self.ij.command().run(ABBABenchMarkCommand, True,
+                                     'comment', comment,
+                                     'demo_dataset', demo_dataset,
+                                     'use_gui', use_gui,
+                                     'wait_betweem_each_step', wait_betweem_each_step).get()
+
+    def check_for_update(self):
+        """
+        Check for updates
+
+        """
+        ABBACheckForUpdateCommand = jimport('ch.epfl.biop.atlas.aligner.command.ABBACheckForUpdateCommand')
+        return self.ij.command().run(ABBACheckForUpdateCommand, True)
+
+    def cite_info(self):
+        """
+        How to cite ABBA
+
+        """
+        ABBACiteInfoCommand = jimport('ch.epfl.biop.atlas.aligner.command.ABBACiteInfoCommand')
+        return self.ij.command().run(ABBACiteInfoCommand, True)
 
     def close(self):
         """
@@ -779,20 +925,20 @@ class Abba:
                                      'grid_spacing_in_micrometer', grid_spacing_in_micrometer).get()
 
     def register_slices_bigwarp(self,
-                                channel_atlas: int,
-                                channel_slice: int):
+                                channels_atlas_csv: str,
+                                channels_slice_csv: str):
         """
         Uses BigWarp for in plane registration of selected slices
 
         Parameters:
-        channel_atlas (int): Atlas channels
-        channel_slice (int): Slices channels
+        channels_atlas_csv (str): Atlas channels (channels comma separated)
+        channels_slice_csv (str): Slices channels (channels comma separated)
         """
         RegisterSlicesBigWarpCommand = jimport('ch.epfl.biop.atlas.aligner.command.RegisterSlicesBigWarpCommand')
         return self.ij.command().run(RegisterSlicesBigWarpCommand, True,
                                      'mp', self.mp,
-                                     'channel_atlas', channel_atlas,
-                                     'channel_slice', channel_slice).get()
+                                     'channels_atlas_csv', channels_atlas_csv,
+                                     'channels_slice_csv', channels_slice_csv).get()
 
     def register_slices_copy_and_apply(self,
                                        model_slice_index: int,
@@ -817,6 +963,7 @@ class Abba:
                                         ensemble: bool,
                                         model: str,
                                         post_processing: str,
+                                        px_size_micron: float,
                                         slices_spacing_micrometer: float):
         """
         Uses Deepslice for affine in plane and axial registration of selected slices
@@ -827,6 +974,7 @@ class Abba:
         ensemble (bool): Average of several models (slower)
         model (str): ('mouse', 'rat') Mouse or Rat ?
         post_processing (str):
+        px_size_micron (float): Resampling pixel size (10 for mouse, 40 for rat)
         slices_spacing_micrometer (float): Spacing (micrometer), used only when 'Keep order + set spacing' is selected
         """
         RegisterSlicesDeepSliceLocalCommand = jimport(
@@ -838,13 +986,15 @@ class Abba:
                                      'ensemble', ensemble,
                                      'model', model,
                                      'post_processing', post_processing,
+                                     'px_size_micron', px_size_micron,
                                      'slices_spacing_micrometer', slices_spacing_micrometer).get()
 
     def register_slices_deepslice_web(self,
                                       allow_slicing_angle_change: bool,
                                       channels: str,
                                       maintain_slices_order: bool,
-                                      model: str):
+                                      model: str,
+                                      px_size_micron: float):
         """
         Uses Deepslice for affine in plane and axial registration of selected slices
 
@@ -853,6 +1003,7 @@ class Abba:
         channels (str): Slices channels, 0-based, comma separated, '*' for all channels
         maintain_slices_order (bool): Keep slices order
         model (str): ('mouse', 'rat') Mouse or Rat ?
+        px_size_micron (float): Resampling pixel size (10 for mouse, 40 for rat)
         """
         RegisterSlicesDeepSliceWebCommand = jimport(
             'ch.epfl.biop.atlas.aligner.command.RegisterSlicesDeepSliceWebCommand')
@@ -861,7 +1012,8 @@ class Abba:
                                      'allow_slicing_angle_change', allow_slicing_angle_change,
                                      'channels', channels,
                                      'maintain_slices_order', maintain_slices_order,
-                                     'model', model).get()
+                                     'model', model,
+                                     'px_size_micron', px_size_micron).get()
 
     def register_slices_edit_last(self,
                                   atlas_channels_csv: str,
@@ -887,7 +1039,7 @@ class Abba:
                                        channels_slice_csv: str,
                                        pixel_size_micrometer: float,
                                        show_imageplus_registration_result: bool = False,
-                                       background_offset_value_moving: float = 0,):
+                                       background_offset_value_moving: float = 0, ):
         """
         Uses Elastix for affine in plane registration of selected slices
 
@@ -914,7 +1066,7 @@ class Abba:
                                        nb_control_points_x: int,
                                        pixel_size_micrometer: float,
                                        show_imageplus_registration_result: bool = False,
-                                       background_offset_value_moving: float = 0,):
+                                       background_offset_value_moving: float = 0, ):
         """
         Uses Elastix for spline in plane registration of selected slices
 
@@ -962,6 +1114,19 @@ class Abba:
                                      'angle_degrees', angle_degrees,
                                      'axis_string', axis_string).get()
 
+    def set_slices_deselected(self,
+                              slices_csv: str):
+        """
+        Set the slices to deselect.
+
+        Parameters:
+        slices_csv (str): Slices to deselect, '*' for all slices, comma separated, 0-based
+        """
+        SetSlicesDeselectedCommand = jimport('ch.epfl.biop.atlas.aligner.command.SetSlicesDeselectedCommand')
+        return self.ij.command().run(SetSlicesDeselectedCommand, True,
+                                     'mp', self.mp,
+                                     'slices_csv', slices_csv).get()
+
     def set_slices_display_range(self,
                                  channels_csv: str,
                                  display_max: float,
@@ -981,6 +1146,19 @@ class Abba:
                                      'display_max', display_max,
                                      'display_min', display_min).get()
 
+    def set_slices_selected(self,
+                            slices_csv: str):
+        """
+        Set the slices to select.
+
+        Parameters:
+        slices_csv (str): Slices to select, '*' for all slices, comma separated, 0-based
+        """
+        SetSlicesSelectedCommand = jimport('ch.epfl.biop.atlas.aligner.command.SetSlicesSelectedCommand')
+        return self.ij.command().run(SetSlicesSelectedCommand, True,
+                                     'mp', self.mp,
+                                     'slices_csv', slices_csv).get()
+
     def set_slices_thickness(self,
                              thickness_in_micrometer: float):
         """
